﻿using System;

namespace School
{


    /// <summary>
    ///     Class invariant: name !=null and name != ""
    /// </summary>
    public class Student : IEquatable<Student>
    {


        private string _name;
        private int _age;

        public Student(String name, int age)
        {
            CheckName(name);
            _name = name;
            _age = age;
        }

        public String Name
        {
            get { return _name; }
            set
            {
                CheckName(value);
                _name = value;
            }
        }

        public int Age
        {
            get { return _age; }
            set
            {
                CheckAge(value);
                _age = value;
            }
        }

        private static void CheckName(string name)
        {
            if (String.IsNullOrEmpty(name))
            {
                throw new ArgumentException("name is null or empty");
            }
        }

        private static void CheckAge(int age)
        {
            if (age < 0)
            {
                throw new ArgumentException("age is less than zero");
            }
        }


        public bool Equals(Student other)
        {
            if (other == null) return false;
            return this._name.Equals(other._name) && this._age == other._age;
        }

        public override bool Equals(Object obj)
        {
            if (obj == null)
                return false;

            Student personObj = obj as Student;
            if (personObj == null)
                return false;
            else
                return Equals(personObj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                return ((_name != null ? _name.GetHashCode() : 0) * 397) ^ _age;
            }
        }


        public override string ToString()
        {
            return string.Format("Student: {0}, {1}", _name, _age);
        }
    }
}